#!/usr/bin/python3
# -*- coding: utf-8 -*-

import random  # Module random pour générer des valeurs aléatoires (avec random.choice)


class PokemonError(Exception):
    ''' Classe permettant de générer des messages d'erreur.
    Par exemple : PokemonError("ERREUR !") '''

    def __init__(self, msg):
        self.message = msg


class Pokemon:
    ''' Classe Pokemon.
    Ses attributs sont :
    - Son nom : Chaîne de caractères
    - Son ou ses types : Tableau de chaînes de caractères (Exemple : ['Feu'] ou ['Normal', 'Air']
    - Son nombre de points de vies : Entier
    - Son attaque : Entier
    - Sa défense : Entier
    - Son état : un dictionnaire, initialement {"nom_etat": "normal", "duree_etat": -1}.
    - Liste de ses attaques sous forme d'une liste de dictionnaires. '''

    def __init__(self, nom, types, pts_vies, attaque, defense, liste_attaques):
        ''' CONSTRUCTEUR A COMPLETER '''
        self._nom = nom
        self._types = types
        self._pts_vies = pts_vies
        self._attaque = attaque
        self._defense = defense
        self._liste_attaques = liste_attaques
        self._etat = {"nom_etat": 'normal', "duree_etat": -1}  # État initial (normal), durée -1 = infinie

    # DEFINITION DES GETTERS :
    # Ce sont les fonctions qui permettent de RECUPERER les valeurs des attributs du pokémon.

    def get_nom(self):
        ''' A COMPLETER - Renvoie la valeur de l'attribut _nom '''
        return self._nom

    def get_vies(self):
        return self._pts_vies

    def get_attaque(self):
        return self._attaque

    def get_defense(self):
        return self._defense

    def get_etat(self):
        return self._etat

    # DEFINITION DES SETTERS :
    # Ce sont les fonctions qui permettent de MODIFIER les valeurs des attributs du pokémon.

    def set_etat(self, n_etat):
        ''' A COMPLETER : Modifie la valeur de l'attribut _etat '''
        self._etat = n_etat
        
    def set_vies(self, n_vies):
        ''' A COMPLETER : Modifie la valeur de l'attribut _pts_vies '''
        self._pts_vies = n_vies
    
    def set_attaque(self, n_attaque):
        ''' A COMPLETER : Modifie la valeur de l'attribut _attaque '''
        self._attaque = n_attaque

    def set_defense(self, n_defense):
        ''' A COMPLETER : Modifie la valeur de l'attribut _defense '''
        self._defense = n_defense

    def baisser_vies(self, nb):
        ''' Fonction qui retire "nb" points de vies au nombre de vies du pokémon.
        Deux possibilités :
        - Si le nouveau nombre de pts de vies est inférieur à 0, on met le nombre de points de vies à 0
        - Sinon, le nouveau nombre de pts de vies est la soustraction de l'ancien nombre de pts de vies et de "nb".
        :param nb: (int) Le nombre de points de vies à retirer. '''

        if self.get_vies() - nb < 0:
            self.set_vies(0)
        else:
            self.set_vies(self.get_vies() - nb)

    def baisser_attaque(self, nb):
        ''' Même principe que baisser_vies() mais pour l'attaque. '''
        if self.get_attaque() - nb < 0:
            self.set_attaque(0)
        else:
            self.set_attaque(self.get_attaque() - nb)

    def baisser_defense(self, nb):
        ''' Même principe que baisser_vies() mais pour la défense. '''
        if self.get_defense() - nb < 0:
            self.set_defense(0)
        else:
            self.set_defense(self.get_defense() - nb)

    def augmenter_vies(self, nb):
        ''' Fonction qui ajoute "nb" vies au nombre de vies actuel du Pokémon. '''
        self.set_vies(self.get_vies() + nb)

    def augmenter_attaque(self, nb):
        self.set_attaque(self.get_attaque() + nb)

    def augmenter_defense(self, nb):
        self.set_defense(self.get_defense() + nb)

    # AUTRES METHODES UTILES

    def est_mort(self):
        ''' Renvoie True si le Pokémon est mort (points de vies à 0), False sinon. '''
        return not self.get_vies()

    # METHODES RELATIVES AUX ATTAQUES

    def choix_attaque_aleatoire(self):
        ''' Renvoie les données d'une attaque choisie aléatoirement.
        COMPLETER LES POINTILLES - On souhaite renvoier un élément de _liste_attaques aléatoire).
        '''

        attaque = random.choice(self._liste_attaques)
        return attaque

    def choix_attaque(self):
        ''' Fonction qui affiche la liste des attaques ainsi que leurs caractéristiques,
        puis renvoie les informations sur l'attaque choisie par le joueur (= un dictionnaire de _liste_attaques).
        COMPLETEZ LES POINTILLES SUR LA DERNIERE LIGNE '''

        i = 1
        for el in self._liste_attaques:  # Pour chaque liste de _liste_attaques (donc pour chaque attaque)
            print(f"{i} - {el['nom_attaque']} : ", end='')  # On affiche un numéro puis le nom de l'attaque
            if len(el) == 3:  # 3 éléments, cela signifie qu'il s'agit d'une attaque spécial (poison, paralysie, etc.)
                if el["nom_effet"] == 'attaque-e':
                    print(f"Diminue de {el['valeur_effet']} l'attaque du pokémon ennemi")
                elif el["nom_effet"] == 'defense-e':
                    print(f"Diminue de {el['valeur_effet']} la défense du pokémon ennemi")
                elif el["nom_effet"] == 'attaque+':
                    print(f"Augmente de {el['valeur_effet']} l'attaque du pokémon allié")
                elif el["nom_effet"] == 'defense+':
                    print(f"Augmente de {el['valeur_effet']} la défense du pokémon allié")
                elif el["nom_effet"] == 'poison':
                    print(f"Empoisonne votre adversaire pour {el['valeur_effet']} tours.")
                elif el["nom_effet"] == 'drainage':
                    print(f"Draine l'énergie de votre adversaire pour {el['valeur_effet']} tours.")
                elif el["nom_effet"] == 'paralysie':
                    print(f"Paralyse votre adversaire pour {el['valeur_effet']} tours.")
                else:
                    PokemonError(f"ERREUR : Type d'attaque {el['nom_effet']} inconnu !")
            else:  # Sinon (2 éléments), il s'agit d'une attaque normale.
                print(f"Puissance de l'attaque : {el['degats_attaque']}")

            i += 1

        choix = input("Quelle attaque choisissez-vous ? ")
        return self._liste_attaques[int(choix) - 1]
